package cn.minbb.evaluationsystemserver.controller;

import cn.minbb.evaluationsystemserver.entity.Logs;
import cn.minbb.evaluationsystemserver.entity.User;
import cn.minbb.evaluationsystemserver.service.LogsService;
import cn.minbb.evaluationsystemserver.service.UserService;
import cn.minbb.evaluationsystemserver.stroage.StorageService;
import cn.minbb.evaluationsystemserver.util.IPUtils;
import cn.minbb.evaluationsystemserver.util.ResponseMap;
import cn.minbb.evaluationsystemserver.util.Result;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;

@Controller
@RequestMapping("/user")
public class UserController {

    @Value("${USER_AVATAR_URL}")
    private String USER_AVATAR_URL;

    private final UserService userService;
    private final StorageService storageService;
    private final LogsService logsService;

    @Autowired
    public UserController(UserService userService, StorageService storageService, LogsService logsService) {
        this.userService = userService;
        this.storageService = storageService;
        this.logsService = logsService;
    }

    @GetMapping(value = "/login")
    @ResponseBody
    public String loginIndex() {
        return "login";
    }

    /**
     * 用户注册
     *
     * @param userJSON
     * @param request
     * @return
     */
    @PostMapping(value = "/register", produces = "text/plain; charset=UTF-8")
    @ResponseBody
    public String register(@RequestBody String userJSON, HttpServletRequest request) {
        ResponseMap responseMap = new ResponseMap();
        try {
            User user = JSONObject.parseObject(userJSON, User.class);
            if (user != null) {
                if (userService.findUserByUsername(user.getUsername()) == null) {
                    User u = userService.save(user);
                    if (u != null) {
                        // 注册成功
                        responseMap.putResult(Result.SUCCESS);
                        responseMap.putSummary("注册成功");
                        responseMap.putData(u);
                        logsService.save(new Logs("用户注册", "注册成功", user, IPUtils.getIPAddress(request)));
                    } else {
                        // 注册失败
                        responseMap.putResult(Result.FAILED);
                        responseMap.putSummary("注册失败");
                        logsService.save(new Logs("用户注册", "注册失败，用户未成功保存", user, IPUtils.getIPAddress(request)));
                    }
                } else {
                    // 用户名已存在
                    responseMap.putResult(Result.FAILED);
                    responseMap.putSummary("用户名已存在");
                    logsService.save(new Logs("用户注册", "注册失败，用户名已存在", user, IPUtils.getIPAddress(request)));
                }
            } else {
                logsService.save(new Logs("用户注册", "错误的注册请求 = " + userJSON, null, IPUtils.getIPAddress(request)));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return JSONObject.toJSONString(responseMap.getMap());
    }

    /**
     * 用户登录
     *
     * @param username 用户名
     * @param password 密码
     * @param request
     * @return
     */
    @PostMapping(value = "/login", produces = "text/plain; charset=UTF-8")
    @ResponseBody
    public String login(
            @RequestParam(value = "username") String username,
            @RequestParam(value = "password") String password,
            HttpServletRequest request) {
        User user = userService.findUserByUsername(username);
        ResponseMap responseMap = new ResponseMap();
        if (user != null) {
            if (user.getPassword().equals(password)) {
                // 密码正确，登录成功
                responseMap.putResult(Result.SUCCESS);
                responseMap.putSummary("身份验证通过");
                // 加密密码
                // user.setPassword("******");
                // Avatar URL
                String userAvatar = user.getAvatar();
                if (userAvatar != null) {
                    user.setAvatar(USER_AVATAR_URL + userAvatar);
                }
                responseMap.putData(user);
                logsService.save(new Logs("用户登录", "登陆成功", user, IPUtils.getIPAddress(request)));
            } else {
                // 密码错误
                responseMap.putResult(Result.FAILED);
                responseMap.putSummary("密码错误");
                logsService.save(new Logs("用户登录", "登陆失败，密码错误", user, IPUtils.getIPAddress(request)));
            }
        } else {
            // 此用户名的用户不存在
            responseMap.putResult(Result.FAILED);
            responseMap.putSummary("用户不存在");
            logsService.save(new Logs("用户登录", "登陆失败，用户名不存在", null, IPUtils.getIPAddress(request)));
        }
        return JSONObject.toJSONString(responseMap.getMap());
    }

    @GetMapping(value = "/avatars/{filename:.+}/download")
    public ResponseEntity<Resource> serveAvatars(@PathVariable String filename) {
        Resource file = storageService.loadAvatarsAsResource(filename);
        return ResponseEntity.ok().header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + file.getFilename() + "\"").body(file);
        // return ResponseEntity.ok().body(file);
    }

    @PostMapping(value = "/alter/avatar", produces = "text/plain; charset=UTF-8")
    @ResponseBody
    public String alterUserAvatar(
            @RequestParam(value = "username") String username,
            @RequestParam(value = "password") String password,
            @RequestParam(value = "avatar", required = true) MultipartFile avatar,
            HttpServletRequest request) {
        User user = userService.findUserByUsername(username);
        ResponseMap responseMap = new ResponseMap();
        if (user != null) {
            if (user.getPassword().equals(password)) {
                // 密码正确
                // 头像名称 = 用户名 + 下划线 + 时间（毫秒） + .文件后缀名
                String avatarName = username + "_" + System.currentTimeMillis() + avatar.getOriginalFilename().substring(avatar.getOriginalFilename().lastIndexOf("."));
                user.setAvatar(avatarName);
                storageService.storeAvatar(avatar, avatarName);
                responseMap.putResult(Result.SUCCESS);
                user = userService.save(user);
                responseMap.putSummary("修改信息成功");
                // 隐藏密码
                // user.setPassword("******");
                // 拼接头像地址
                String userAvatar = user.getAvatar();
                if (userAvatar != null) {
                    user.setAvatar(USER_AVATAR_URL + userAvatar);
                }
                responseMap.putData(user);
                logsService.save(new Logs("用户修改头像", "修改成功，新头像地址 = " + user.getAvatar(), user, IPUtils.getIPAddress(request)));
            } else {
                // 密码错误
                responseMap.putResult(Result.FAILED);
                responseMap.putSummary("密码错误");
                logsService.save(new Logs("用户修改头像", "修改失败，密码错误", user, IPUtils.getIPAddress(request)));
            }
        } else {
            // 此用户名的用户不存在
            responseMap.putResult(Result.FAILED);
            responseMap.putSummary("用户不存在");
            logsService.save(new Logs("用户修改头像", "修改失败，用户名不存在", null, IPUtils.getIPAddress(request)));
        }
        return JSONObject.toJSONString(responseMap.getMap());
    }

    @PostMapping(value = "/alter/info", produces = "text/plain; charset=UTF-8")
    @ResponseBody
    public String alterUserInfo(
            @RequestParam(value = "username") String username,
            @RequestParam(value = "password") String password,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "email", required = false) String email,
            @RequestParam(value = "phone", required = false) String phone,
            @RequestParam(value = "qq", required = false) String qq) {
        User user = userService.findUserByUsername(username);
        ResponseMap responseMap = new ResponseMap();
        if (user != null) {
            if (user.getPassword().equals(password)) {
                // 密码正确
                if (name != null) {
                    user.setName(name);
                }
                if (email != null) {
                    user.setEmail(email);
                }
                if (phone != null) {
                    user.setPhone(phone);
                }
                if (qq != null) {
                    user.setQq(qq);
                }
                responseMap.putResult(Result.SUCCESS);
                user = userService.save(user);
                responseMap.putSummary("修改信息成功");
                // 隐藏密码
                // user.setPassword("******");
                // Avatar URL
                String userAvatar = user.getAvatar();
                if (userAvatar != null) {
                    user.setAvatar(USER_AVATAR_URL + userAvatar);
                }
                responseMap.putData(user);
            } else {
                // 密码错误
                responseMap.putResult(Result.FAILED);
                responseMap.putSummary("密码错误");
            }
        } else {
            // 此用户名的用户不存在
            responseMap.putResult(Result.FAILED);
            responseMap.putSummary("用户不存在");
        }
        return JSONObject.toJSONString(responseMap.getMap());
    }

    @PostMapping(value = "/alter/password", produces = "text/plain; charset=UTF-8")
    @ResponseBody
    public String alterUserInfo(
            @RequestParam(value = "username") String username,
            @RequestParam(value = "oldPassword") String oldPassword,
            @RequestParam(value = "newPassword") String newPassword
    ) {
        return "ok";
    }

    @PostMapping(value = "/alter", produces = "text/plain; charset=UTF-8")
    @ResponseBody
    public String alterAvatar(@RequestBody String s) {
        System.out.println(s);
        return "ok";
    }

    @PostMapping(value = "/add", produces = "text/plain; charset=UTF-8")
    public String handleAddHeroPage(
            @RequestParam("name") String name,
            @RequestParam("category") String category,
            @RequestParam("picture") MultipartFile picture) {
        storageService.store(picture);
        return "redirect:/";
    }
}
