package cn.minbb.evaluationsystemserver.entity;

import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.UpdateTimestamp;

import javax.persistence.*;
import javax.persistence.Version;
import java.util.Date;
import java.util.Set;

@Entity
@Table(name = "message")
@NoArgsConstructor
public class Message {

    @Getter
    @Setter
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "e_id", nullable = false, columnDefinition = "BIGINT COMMENT '消息实体ID'")
    private Long e_id;

    @Getter
    @Setter
    @Column(name = "content", nullable = false, columnDefinition = "VARCHAR(255) COMMENT '消息内容'")
    private String content;

    @Getter
    @Setter
    @Column(name = "type", nullable = false, columnDefinition = "TINYINT DEFAULT 0 COMMENT '消息类型'")
    private Integer type;

    @Getter
    @Setter
    @Column(name = "status", nullable = false, columnDefinition = "TINYINT DEFAULT 0 COMMENT '接收状态'")
    private Boolean status;

    @Getter
    @Setter
    @ManyToOne(cascade = {CascadeType.REFRESH}, optional = false)
    @JoinColumn(name = "user_from", columnDefinition = "BIGINT COMMENT '消息发送者用户实体ID'")
    private User userFrom;

    @Getter
    @Setter
    @ManyToMany(mappedBy = "messageSetTo")
    // 消息接收者（集合）
    private Set<User> userTo;

    @Getter
    @Setter
    @Column(name = "created_at", columnDefinition = "DATETIME COMMENT '创建时间'")
    @CreationTimestamp
    private Date createdAt;

    @Getter
    @Setter
    @Column(name = "updated_at", columnDefinition = "DATETIME COMMENT '更新时间'")
    @UpdateTimestamp
    private Date updatedAt;

    @Version
    @Column(name = "version", columnDefinition = "BIGINT COMMENT '版本号'")
    public Long version;
}
