package cn.minbb.evaluationsystemserver.entity;

import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.UpdateTimestamp;

import javax.persistence.*;
import javax.persistence.Version;
import java.util.Date;
import java.util.List;

@Entity
@Table(name = "paper")
@NoArgsConstructor
public class Paper {

    @Getter
    @Setter
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "e_id", nullable = false, columnDefinition = "BIGINT COMMENT '试卷实体ID'")
    private Long e_id;

    @Getter
    @Setter
    // 若 optional = false 就代表 user 的值是必须存在的，反映在数据库中的外键字段的值就是禁止为空
    @ManyToOne(cascade = {CascadeType.MERGE, CascadeType.REFRESH}, optional = false)
    // 指定外键的名称，外键一般都是在关系维护端定义
    @JoinColumn(name = "user_id", columnDefinition = "BIGINT COMMENT '用户实体ID'")
    private User user;

    @Getter
    @Setter
    @Column(name = "score", nullable = false, columnDefinition = "FLOAT DEFAULT 0 COMMENT '试卷总分值'")
    private Float score;

    @Getter
    @Setter
    @Column(name = "mark", columnDefinition = "FLOAT DEFAULT 0 COMMENT '试卷得分'")
    private Float mark;

    @Getter
    @Setter
    @Column(name = "started_at", nullable = false, columnDefinition = "DATETIME COMMENT '开始时间'")
    private Date startedAt;

    @Getter
    @Setter
    @Column(name = "ended_at", columnDefinition = "DATETIME COMMENT '结束时间'")
    private Date endedAt;

    @Getter
    @Setter
    @ManyToMany(cascade = CascadeType.REFRESH)
    @JoinTable(name = "paper_question",
            joinColumns = {@JoinColumn(name = "paper_id", referencedColumnName = "e_id", columnDefinition = "BIGINT COMMENT '试卷实体ID'")},
            inverseJoinColumns = {@JoinColumn(name = "question_id", referencedColumnName = "e_id", columnDefinition = "BIGINT COMMENT '题目实体ID'")})
    // 题目（列表）
    private List<Question> questionList;

    @Getter
    @Setter
    @ManyToOne(cascade = {CascadeType.REFRESH}, optional = true)
    @JoinColumn(name = "reply_id", columnDefinition = "BIGINT COMMENT '回复实体ID'")
    private Reply reply;

    @Getter
    @Setter
    @Column(name = "created_at", columnDefinition = "DATETIME COMMENT '创建时间'")
    @CreationTimestamp
    private Date createdAt;

    @Getter
    @Setter
    @Column(name = "updated_at", columnDefinition = "DATETIME COMMENT '更新时间'")
    @UpdateTimestamp
    private Date updatedAt;

    @Version
    @Column(name = "version", columnDefinition = "BIGINT COMMENT '版本号'")
    public Long version;

    public Paper(User user, Float score, List<Question> questionList) {
        this.user = user;
        this.score = score;
        this.questionList = questionList;
    }
}
