package cn.minbb.evaluationsystemserver.entity;

import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.UpdateTimestamp;

import javax.persistence.*;
import javax.persistence.Version;
import java.util.Date;
import java.util.List;
import java.util.Set;

@Entity
@Table(name = "question")
@NoArgsConstructor
public class Question {

    @Getter
    @Setter
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "e_id", nullable = false, columnDefinition = "BIGINT COMMENT '题目实体ID'")
    private Long e_id;

    @Getter
    @Setter
    @Column(name = "type", nullable = false, columnDefinition = "INT DEFAULT 1 COMMENT '题目类别'")
    private Integer type;

    @Getter
    @Setter
    @Column(name = "content", nullable = false, columnDefinition = "VARCHAR(255) COMMENT '题目内容'")
    private String content;

    @Getter
    @Setter
    @Column(name = "explanation", columnDefinition = "VARCHAR(255) COMMENT '题目解答'")
    private String explanation;

    @Getter
    @Setter
    @Column(name = "score", nullable = false, columnDefinition = "FLOAT DEFAULT 0 COMMENT '题目分值'")
    private Float score;

    @Getter
    @Setter
    @OneToMany(cascade = {CascadeType.REFRESH}, mappedBy = "question", fetch = FetchType.EAGER)
    // 题目答案（集合）
    private Set<Answer> answerSet;

    @Getter
    @Setter
    @Column(name = "enabled", nullable = false, columnDefinition = "TINYINT DEFAULT 1 COMMENT '启用题目'")
    private Boolean enabled;

    @Getter
    @Setter
    @ManyToMany(mappedBy = "questionList")
    // 题目类别（列表）
    private List<Classification> classificationList;

    @Getter
    @Setter
    @ManyToMany(mappedBy = "questionList")
    // 试卷（列表）
    private List<Paper> paperList;

    @Getter
    @Setter
    @Column(name = "created_at", columnDefinition = "DATETIME COMMENT '创建时间'")
    @CreationTimestamp
    private Date createdAt;

    @Getter
    @Setter
    @Column(name = "updated_at", columnDefinition = "DATETIME COMMENT '更新时间'")
    @UpdateTimestamp
    private Date updatedAt;

    @Version
    @Column(name = "version", columnDefinition = "BIGINT COMMENT '版本号'")
    public Long version;
}
