package cn.minbb.evaluationsystemserver.websocket;

import cn.minbb.evaluationsystemserver.entity.Message;
import cn.minbb.evaluationsystemserver.entity.User;
import cn.minbb.evaluationsystemserver.service.MessageService;
import cn.minbb.evaluationsystemserver.service.UserService;
import cn.minbb.evaluationsystemserver.util.App;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.handler.annotation.DestinationVariable;
import org.springframework.messaging.handler.annotation.MessageMapping;
import org.springframework.messaging.handler.annotation.SendTo;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.messaging.simp.annotation.SubscribeMapping;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.socket.config.WebSocketMessageBrokerStats;

@Controller
@RequestMapping("")
public class WebSocketController {

    public static SimpMessagingTemplate template;
    private UserService userService;
    private MessageService messageService;
    private final WebSocketMessageBrokerStats webSocketMessageBrokerStats;

    @Autowired
    public WebSocketController(SimpMessagingTemplate template, UserService userService, WebSocketMessageBrokerStats webSocketMessageBrokerStats) {
        WebSocketController.template = template;
        this.userService = userService;
        this.webSocketMessageBrokerStats = webSocketMessageBrokerStats;
        new Thread(() -> {
            while (true) {
                try {
                    Thread.sleep(2000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                Message messageTest = new Message();
                messageTest.setContent(App.getCurrentStringTime());
                messageTest.setCreatedAt(App.getCurrentDateTime());
                template.convertAndSend("/topic/chat/all", JSONObject.toJSONString(messageTest));
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                messageTest.setContent(webSocketMessageBrokerStats.getWebSocketSessionStatsInfo());
                template.convertAndSend("/topic/chat/all", JSONObject.toJSONString(messageTest));
            }
        }).start();
    }

    @MessageMapping("/online/{id}")
    public void online(@DestinationVariable("id") Long id) throws InterruptedException {
        // 上线广播
        Thread.sleep(3 * 1000);
        template.convertAndSend("/queue/online/" + id, JSON.toJSONString("欢迎登录"));
    }

    @MessageMapping("/chat/all")
    public void chatPublic(String message) {
        // 公共聊天频道
        Message messageClient = JSONObject.parseObject(message, Message.class);
        if (messageClient != null) {
            User userClient = messageClient.getUserFrom();
            if (userClient != null) {
                User userServer = userService.findUserByUsername(userClient.getUsername());
                if (userServer != null && userClient.getPassword().equals(userServer.getPassword())) {
                    messageClient.setType(8);
                    messageClient.setUserTo(null);
                    Message messageServer = messageService.save(messageClient);
                    template.convertAndSend("/topic/chat/all", JSONObject.toJSONString(messageServer));
                } else {
                    // 消息发送者身份验证失败
                    System.out.println("消息发送者身份验证失败");
                }
            } else {
                // 消息发送者不能为空
                System.out.println("消息发送者不能为空");
            }
        }
    }

    @MessageMapping("/m/1")
    // @SendTo("/topic/1")
    // @DestinationVariable("user") User userJSON
    public void broadcast() {
        // 广播
        template.convertAndSend("/topic/1", JSON.toJSONString("欢迎使用 m1"));
    }

    @MessageMapping("/m/2")
    @SendTo("/topic/2")
    public String broadcast2() throws InterruptedException {
        Thread.sleep(1000);
        return "欢迎使用 m2";
    }

    @SubscribeMapping("/s/1")
    public String subscribe() throws InterruptedException {
        Thread.sleep(3000);
        return "感谢订阅";
    }
}
